# Duo, A crazy eights game.
# Copyright (C) 2004,2009 The Duo developers.
#
# This file is part of Duo.
#
# Duo is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
The first robot written for Duo.

Camille Roux is the initial author of Duo. When the code was revamped
to separate roles, the part of the program taking care of the
computer's controlled players was used to create the first AI and was
named Camille.

Camille used to play with the regular Uno rules. It was looking for
cards matching the heap type or the heap color, and then a
jocker. When playing a jocker, it was choosing the color the most
present in its hand.

With the isolation of the game rules, Camille didn't change its
habits. It searches in the set of possible actions what are the card
moves which do not imply a joker first. And if playing a joker, it
still computes the better color to choose.
"""
from __future__ import with_statement

import logging
import threading
import time

from duo import ai
from duo import game
from duo import protocol
from duo import referee

NAMES = [ 'Camille', 'Calvin', 'Castor', 'Caterina', 'Cecil', 'Chantal',
          'Chen', 'Christen', 'Clovis', 'Consuela', 'Cvetko', 'Cymbeline' ]

logger = logging.getLogger('duo.ai.camille')


class AI(ai.AbstractAI):
    def __init__(self):
        ai.AbstractAI.__init__(self)
        logger.debug('AI.__init__')
        self._condition = threading.Condition()
        self._end = False

    def stop_game(self):
        with self._condition:
            self._end = True
            self._condition.notify()

    def on_hand(self, message, emitter, args):
        with self._condition:
            self._condition.notify()

    def on_winner(self, payload, emitter, args):
        with self._condition:
            logger.debug(
                '%s congratulates %s...' % (self.player.name, payload))
            self._end = True
            self._condition.notify()

    def run(self):
        """TODO"""
        logger.debug('%s is back at the table.' % self.player.name)
        self._condition.acquire()
        while not self._end:
            # Wait for our turn.
            self._condition.wait()

            # Issue a move.
            hand = self.player.hand
            actions = self.player.actions

            if not self._end and len(actions):
                action = self._play(actions, hand)
                self.player.send_to_referee(protocol.ActionMessage(action))

        logger.debug('%s asks for a drink.' % self.player.name)
        self._condition.release()

        # Reset between rounds.
        self._end = False

    def _play(self, actions, hand):
        """TODO"""
        logger.debug('%s is thinking...' % self.player.name)

        time.sleep(1)

        if len(actions) == 1:
            return actions.pop()

        else:
            # Try to play normal card, then a wild one.
            # Play randomly in last resort.
            card_actions = [action for action in actions
                            if isinstance(action, referee.CardAction)]

            if len(card_actions):
                # Search a colored card.
                for action in card_actions:
                    if action.card.color != 'N':
                        return action

                # Search for a preferred wild card.
                color = self._get_preferred_color(hand)
                for action in card_actions:
                    if action.color == color:
                        return action

            # Finally, play at random.
            return actions.pop()

    def _get_preferred_color(self, hand):
        """choose the color to ask"""
        chosen_color = 'B'
        max = 0
        colors = [card.color for card in hand
                  if card.color !='N']
        for color in game.Card.COLORS.keys():
            count = colors.count(color)
            if count > max:
                max = count
                chosen_color = color
        logger.debug('%s chooses %s' % (self.player.name, chosen_color))
        return chosen_color
